#!/usr/bin/env python
#########################################################
#              RETicker v. 0.4.4
# a programm to display xml-formatted rss/rdf-feeds in
# a one line display
#########################################################
# please read Help.txt for contact information

#########################################################
# This file contains the classes storing and showing    #
# the actual news                                       #
#########################################################
from __future__ import generators
# import findrox; findrox.version(1,9,12)
# it uses a patched version of ROX-Lib2 in rox/
import rox
from rox import g, tasks

from xml.dom import minidom
import xml.sax
import urllib

import os
import time

import tool
from Options import *
global v # be verbose
v = None

### old imports: mightbe deleted sometimes ;)
#from rox.options import Option
#import overview	
#import sys
#from rox.Menu import Menu, set_save_name
#import gc
#import re

### class for a news message and all information related to one
class message:
	def __init__(self, title, link, desc, time, fetchtime, showcount, read):
		self.title = title
		self.link  = link
		self.desc  = desc
		self.time  = time		# time wich found in field dc:date
		self.fetchtime = fetchtime	# time when message was fetched
		self.showcount = showcount	# how often message was shown in ticker
		self.read = read		# if 1, message was opened in browser
	### export contents of message to xml
	def _to_xml(self, parent):
		def createNode(doc, name, data):
			node = doc.createElement(name)
			node.appendChild(doc.createTextNode(data))
			return node
		doc = parent.ownerDocument
		root = doc.createElement('item')
		parent.appendChild(root)
		root.appendChild(createNode(doc, 'title', self.title))
		root.appendChild(createNode(doc, 'link', self.link))
		root.appendChild(createNode(doc, 'description', self.desc))
		root.appendChild(createNode(doc, 'time', str(self.time)))
		root.appendChild(createNode(doc, 'fetchtime', str(self.fetchtime)))
		root.appendChild(createNode(doc, 'showcount', str(self.showcount)))
		root.appendChild(createNode(doc, 'read', str(self.read)))
		return doc
				
### class for a news feed -- contains a list of messages 
class feed:
	def __init__(self, title, active, ticker, icon, iconcache, \
			shownews, news_age_by_date, link, desc):
		### data-fields with basic information
		self.title = title
		self.active = active
		self.ticker = ticker
		self.icon = icon
		self.iconcache = iconcache
		self.shownews = shownews
		self.news_age_by_date = news_age_by_date
		self.link = link
		self.desc = desc
		self.msg = []			# stores the messages
		### additional fields
		self.valid = 1			# used in convert_sources to decide,
						# if feed is still valid
		self.date_info = 0		# feed provides dc:date-information
		self.add_pos = 0		# pos to add next msg
		self.cache_count = 0		# max number of msg to cache
		self.tmp_cache_count = 0	
	### export contents of feed and all messages to xml
	def _to_xml(self, parent):
		def createNode(doc, name, data):
			node = doc.createElement(name)
			node.appendChild(doc.createTextNode(data))
			return node
		doc = parent.ownerDocument
		root = doc.createElement('feed')
		parent.appendChild(root)
		root_c = doc.createElement('channel')
		root.appendChild(root_c)
		root_c.appendChild(createNode(doc, 'title', self.title))
		root_c.appendChild(createNode(doc, 'active', str(self.active)))
		root_c.appendChild(createNode(doc, 'ticker', self.ticker))
		root_c.appendChild(createNode(doc, 'icon', self.icon))
		root_c.appendChild(createNode(doc, 'iconcache', self.iconcache))
		root_c.appendChild(createNode(doc, 'shownews', str(self.shownews)))
		root_c.appendChild(createNode(doc, 'news_age_by_date', str(self.news_age_by_date)))
		root_c.appendChild(createNode(doc, 'link', self.link))
		root_c.appendChild(createNode(doc, 'description', self.desc))
		for i in self.msg:
			i._to_xml(root)
		return doc		
	### if number of messages from server increases, inc cache_count
	### remove last msg as long as to many msg stored
	def pop_cache(self):
		if self.cache_count < self.tmp_cache_count:
			self.cache_count = self.tmp_cache_count
		while len(self.msg) > self.cache_count: self.msg.pop()
	### bevor refreshing this feed, reset some counters
	def reinit(self):
		self.add_pos = 0
		self.tmp_cache_count = 0
	### say if two messages are equal (here same title and same link)
	def cmp_msg(self, msg1, msg2):
		#print "c: " + msg1.title[:10] + " vs " + msg2.title[:10]
		if msg1.title <> msg2.title: return False	
		if msg1.link <> msg2.link: return False
		return True
	### adds message, if it is a new message
	def add_msg(self,msg_new):
		self.tmp_cache_count += 1
		for msg_old in self.msg:
			# try, if message still in feed
			if self.cmp_msg(msg_old, msg_new): return
		if msg_new.time <> 0: # feed-msg provides dc:date info, so use it
				      # to find right position for new msg
			self.date_info = 1 # indicate dc:date info
			#search timeposition
			for index in range(0, len(self.msg)-1):
				index_time = self.msg[index].time
				if index_time == 0: index_time = self.msg[index].fetchtime
				if index_time < msg_new.time:
					self.msg.insert(index, msg_new)
					if index < self.add_pos: self.add_pos += 1
					return
			self.msg.append(msg_new)	#not found
		else:
			# if no cd_timestamp provided, add at first position
			self.msg.insert(self.add_pos, msg_new)
			self.add_pos += 1
	### check, if message is new in terms of user wishes
	def msg_is_new(self, msg):
		# check if message was read in browser
		if msg.read == 1: return False
		# check if msg-fetchtimestamp is to old
		if news_age.int_value <> 0 and \
			msg.fetchtime + news_age.int_value*60 < time.time():
			return False
		# check if msg-timestamp is to old
		if self.news_age_by_date <> 0 and self.date_info == 1 and \
			msg.time + self.news_age_by_date * 60 < time.time():
			return False
		# check if msg-showcount is less then max showcount
		if news_showcount.int_value <> 0 and \
			msg.showcount >= news_showcount.int_value:
			return False
		return True
	
class show_list:
	def __init__(self, news, box):
		self.news = news
		self.box = box
		self.fd_count = 0
		self.msg_index = 0
		self.feed_jmp = 3 # init-wert
		self.error = ""
		self.updatetimeout = None
		tasks.Task(self.init_wait_task())
	def is_overview_feed(self, feed_nr):
		if self.news.feeds[feed_nr].active == 1:
			self.msg_index = 0
			for msg in self.news.feeds[feed_nr].msg:
				### nur Ticker parallel updaten, wenn im neuen feed neue news sind
				if self.news.feeds[feed_nr].msg_is_new(msg):
					self.fd_count = feed_nr
					self.feed_jmp = 3
					self.update_ticker()
					return True
				if tt_a_fd_wo_news.int_value == 1: return True
		if tt_a_inactive_fd.int_value == 1: return True
		return False
	def next_feed(self, feed_nr):
		while feed_nr < len(self.news.feeds)-1:
			feed_nr += 1
			if self.is_overview_feed(feed_nr): return feed_nr
		if self.no_msg(): return None
		else:
			feed_nr = -1
			return self.next_feed(feed_nr)
	def prev_feed(self, feed_nr):
		while feed_nr <> 0:
			feed_nr -= 1
			if self.is_overview_feed(feed_nr): return feed_nr
		if self.no_msg(): return None
		else:
			feed_nr = len(self.news.feeds)
			return self.prev_feed(feed_nr)
	def no_msg(self):
		self.error = "no messages"
		for fd in self.news.feeds:
			if fd.active == 1 and len(fd.msg) > 0 and \
				fd.shownews > 0:
				#if news_age.int_value == 0: return False
				self.error = "only old messages"
				count = 1
				for i in fd.msg:
					if fd.msg_is_new(i): return False
					count += 1
					if count > fd.shownews: break
		return True
	def next_msg(self):
		# sind news da oder ist lokaler Showcount berschritten?
		self.error = ""
		if self.feed_jmp == 1:
			self.msg_index = 0
			if self.fd_count + 1 < len(self.news.feeds):
				self.fd_count = self.fd_count + 1
				import RETicker
				tasks.Task(RETicker.main.overview.akt_feed())
			else:
				if self.no_msg(): return None
				self.fd_count = 0
		else:
			if self.feed_jmp <> 3: self.msg_index += 1
		self.feed_jmp = 0
		if self.akt_feed().active == 1:
			while (self.akt_feed().shownews > self.msg_index) and \
				(len(self.akt_feed().msg) > self.msg_index):
				if self.akt_msg_is_new(): 
					self.akt_msg().showcount += 1
					return self.akt_msg()
				self.msg_index += 1
		# check next feed
		self.feed_jmp = 1
		return self.next_msg()
	def akt_feed(self):
		if self.news.feeds <> []: return self.news.feeds[self.fd_count]
		else: return None
	def akt_msg(self):
		return self.akt_feed().msg[self.msg_index]
	def akt_msg_is_new(self):
		return self.akt_feed().msg_is_new(self.akt_msg())
	def init_wait_task(self): #wartet, bis News da sind
		while len(self.news.feeds) == 0:
			yield tasks.TimeoutBlocker(2)
		self.update_ticker()
	def add_update(self):
		if self.updatetimeout == None:
			#print "update_timeout added"
			self.updatetimeout = g.timeout_add(1000*update_time.int_value, self.update_ticker)
	def rem_update(self):
		if self.updatetimeout <> None:
			#print "update_timeout removed"
			g.timeout_remove(self.updatetimeout)
			self.updatetimeout = None
	def update_ticker(self):
		#print "vor update ticker (" + self.news.stat() + ")"
		#entferne timeout, damit nicht zwei Tasks gleichzeitig updaten
		self.rem_update()
		if v: print "update Ticker"
	       	
		#if self.update_ticker_(): self.news.export_xml() # wird im Moment nur bei autom. Tickerupdate
		self.update_ticker_()
		
		#fge Timeout wieder hinzu
		self.add_update()
		return True
	
	def update_ticker_(self):
		#print "update ticker" + str(len(self.news.feeds)) + self.news.stat()
		i = self.next_msg()
		if not(i):
			time_to_wait = int(refresh_time.int_value - (time.time() - self.news.lastrefresh)/60)
			if time_to_wait == 0:
				time_to_wait_s = str(int((refresh_time.int_value - (time.time() - self.news.lastrefresh)/60)*60)) + " seconds"
			else:
				time_to_wait_s = str(time_to_wait) + " minutes"
			self.box.label.update(self.error + ", waiting " + time_to_wait_s)
			self.box.label.tooltip.disable()
			self.box.image.set_from_stock(g.STOCK_CANCEL)
	       		self.box.image.tooltip.disable()
			return False
		else:
			fd = self.news.feeds[self.fd_count]
			name = tool.unxml(i.title).encode("utf-8")
			#print name
			self.box.label.update(name)
			self.box.image.get_from_file(fd.iconcache)
			self.box.label.addTip(i)
			self.box.image.addTip(fd, self.news)
			return True
	def open_page(self):
		self.box.status_1("opening page in browser", g.STOCK_CONVERT); yield None
		self.akt_msg().read = 1
		if br_options.value <> "":
			os.spawnlp(os.P_NOWAIT,browser.value,browser.value,br_options.value,self.akt_msg().link) 
		else: 	os.spawnlp(os.P_NOWAIT,browser.value,browser.value,self.akt_msg().link)

	def open_page_feed(self):
		self.box.status_1("opening feed-page in browser", g.STOCK_CONVERT); yield None
		if br_options.value <> "":
			os.spawnlp(os.P_NOWAIT,browser.value,browser.value,br_options.value,self.akt_feed().link) 
		else: 	os.spawnlp(os.P_NOWAIT,browser.value,browser.value,self.akt_feed().link)
		
class news_:
	def __init__(self, box):
		self.feeds = []
		self.init = 0 # registriert, ob gecachte Daten gelesen werden
		self.box = box
		tasks.Task(self.import_xml())
		self.refresh_possible = True
		self.refreshtimeout = None
		self.lastrefresh = time.time()
	def get_by_ticker(self, ticker):
		for fd in self.feeds:
			if ticker == fd.ticker: return fd
	def cmp_feed(self, fd1, fd2):
		if fd1.ticker <> fd2.ticker:
			#print "ungleich"
			return False
		if fd1.valid == 1:
			#print "valid"
			return False
		#print "gleich und nicht valid"
		fd1.valid = 1
		fd1.active = fd2.active
		fd1.icon = fd2.icon
		fd1.iconcache = fd2.iconcache
		fd1.shownews = fd2.shownews
		fd1.news_age_by_date = fd2.news_age_by_date
		return True
	def stat(self):
		msg_count = 0
		allmsg_count = 0
		new_count = 0
		act_count = 0
		for fd in self.feeds:
			#print "ticker: " + str(fd.ticker)
			if fd.active == 1:
				act_count += 1
				msg_count += len(fd.msg)
				i = 0
				for msg in fd.msg:
					if i < fd.shownews:
						if fd.msg_is_new(msg): new_count += 1
						i += 1
					else: continue	
			allmsg_count += len(fd.msg)
		stat =  "feeds: " + str(len(self.feeds)) + "[all]/" + str(act_count) + "[active] msg:" \
				+ str(allmsg_count) + "[all]/"+ str(msg_count) + "[active]/"+ str(new_count) + "[new]"
		return stat
	def export_xml(self):
		from xml.dom.minidom import Document
		doc = Document()
		root = doc.createElement('news')
		doc.appendChild(root)
		for i in self.feeds:
			#print str(i)
			i._to_xml(root)
		#print doc.toxml()
		if not xml_cache.value:
			return	# Saving is disabled
		file = open(xml_cache.value, 'w')
		doc.writexml(file)
		file.close()
		print "Export xml done      (" + self.stat() + ")"
	def import_xml(self):
		if v: print "import cached news ..."
		if not os.path.isfile(xml_cache.value):
			self.init = 1
			return
		self.box.status_1("import cached news ...", g.STOCK_CONVERT)
		yield None
		try:
			doc=minidom.parse(xml_cache.value)
		except xml.sax.SAXParseException:
			self.box.status_1("parse error in cached news ...", g.STOCK_CANCEL)
			yield tasks.TimeoutBlocker(2);
			self.init = 1
			return
		root = doc.documentElement
		assert root.localName == 'news'
		for o in root.childNodes:
			self.read_xml(None, o)
		self.init = 1 # init done
		if v: print "Cached news imported (" + self.stat() + ")"
		print "Cached news imported (" + self.stat() + ")"
		import RETicker
		RETicker.main.overview.create_window(RETicker.main.show_list)
		tasks.Task(self.refresh_ticker())
	def read_xml(self, fd, xmldoc):
		channels=xmldoc.getElementsByTagName("channel")
		channel = channels[0]
		fd_title = tool.getdata(channel, "title")
		ticker = tool.getdata(channel, "ticker")
		link = tool.getdata(channel, "link")
		desc = tool.getdata_wo_ret(channel, "description")
		if self.init == 0:
			active = tool.getintdata(channel, "active")
			icon  = tool.getdata(channel, "icon")
			iconcache = tool.getdata(channel, "iconcache")
			shownews = tool.getintdata(channel, "shownews")
			news_age_by_date = tool.getintdata(channel, "news_age_by_date")
			fd = feed(fd_title, active, ticker, icon, iconcache,\
				shownews, news_age_by_date, link, desc)
			#print "feed added"
			self.add_feed(fd)
		else:
			if fd_title <> "": fd.title = fd_title
			if link <> "": fd.link = link
			if desc <> "": fd.desc = desc
		items=xmldoc.getElementsByTagName("item")
		#yield None
		fd.reinit()
		site_count = 0
		for i in items:
			self.box.status("Checking " + fd_title + " (" + str(site_count) + ")");
			#yield None
			title = tool.getdata(i, "title")
			# manchmal ist der Ticker nicht geupdated, dann msg bergehen
			if title == "": continue
			link = tool.getdata(i, "link")
			desc = tool.getdata_wo_ret(i, "description")
			if desc == "": desc="(keine weitere Info)"
			if self.init == 0:
				time_ = tool.getfloatdata(i, "time")
				fetchtime = tool.getfloatdata(i, "fetchtime")
			else:
				time_ = tool.gettimedata(i, "dc:date")
				fetchtime = time.time()
			showcount = tool.getintdata(i, "showcount")
			read =  tool.getintdata(i, "read")
			tmp_msg = message(title, link, desc, time_, fetchtime, showcount, read)
			fd.add_msg(tmp_msg)
			#print str(fd_title) + " enthlt " + str(len(fd.msg)+1) + " Messages"
			site_count += 1
		fd.pop_cache()
		#self.wait.trigger()
	def add_feed(self,fd_new):
		#print len(self.feeds)
		for fd_indx in range(0, len(self.feeds)):
			# prfe, ob feed schon vorhanden
			if self.cmp_feed(self.feeds[fd_indx], fd_new):
				return fd_indx
		#print "anhngen" 
		self.feeds.append(fd_new)
		return len(self.feeds)-1
	def unvalid_all(self):
		for fd in self.feeds:
			fd.valid = 0
	def clean(self):
		i = 0
		while i < len(self.feeds):
			if self.feeds[i].valid == 0:
				self.feeds.pop(i)
			else: i += 1
		#print "after clean (" + self.stat() + ")"
	def convert_sources(self):
		doc = minidom.parseString(tool.unxml(sources.value))
		Index = 0
		self.unvalid_all()
		for value in doc.getElementsByTagName("source"):
			active = tool.getintdata(value, "active")
			ticker = tool.getdata(value, "ticker")
			if ticker.find("://") == -1: ticker = "http://" + ticker
			icon = tool.getdata(value, "icon")
			if icon == "":
				pos = ticker[8:].find("/")
				if pos <> -1: pos = pos + 8
				if pos <> -1: icon =  ticker[:pos] + "/favicon.ico"
				else:	icon =  ticker + "/favicon.ico"
			else:
				if icon.find("://") == -1: icon = "http://" + icon
			iconcache = tool.getdata(value, "iconcache")
			if iconcache == "":
				iconcache = tmp_name(icon_cache_dir.value, icon)
			shownews = tool.getintdata(value, "shownews")
			news_age_by_date = tool.getintdata(value, "news_age_by_date")
			new_feed = feed("", active, ticker, icon, iconcache,\
					shownews, news_age_by_date, "", "")
			#print "in convert (" + self.stat() + ")"
			self.add_feed(new_feed)
		self.clean()
	def add_refresh(self):
		if self.refreshtimeout == None:
			#print "refresh_timeout added"
			self.refreshtimeout = g.timeout_add(60000*refresh_time.int_value, self.start_refresh_ticker_task)
	def rem_refresh(self):
		if self.refreshtimeout <> None:
			#print "refresh_timeout removed"
			g.timeout_remove(self.refreshtimeout);
			self.refreshtimeout = None
	def start_refresh_ticker_task(self):
		tasks.Task(self.refresh_ticker())
	def refresh_ticker(self):
		if self.refresh_possible:
			self.refresh_possible = False
		else:
			if v: print "refresh Ticker blocked"
			return
		self.rem_refresh()
		if v: print "refresh Ticker"
		self.lastrefresh = time.time()
		self.convert_sources()
		if v: print "found " + str(len(self.feeds)) + " sources"
		for i in self.feeds:
			if i.active <> 1: continue
			if not(os.path.isfile(i.iconcache)):
				icon = i.icon
				iconcache = i.iconcache
				if v: print "Getting Icon from " + icon
				self.box.status_1("Getting Icon from " + icon, g.STOCK_JUMP_TO)
				yield None
				try:
					urllib.urlretrieve(icon,iconcache)
				except (IOError, EOFError):
					iconcache = rox.app_dir+"/AppIcon.xpm"
					self.box.status_1("Icon at " + icon + " not retrieved",g.STOCK_CANCEL)
					yield tasks.TimeoutBlocker(2)
		tasks.Task(self.check_feed())
	
	def check_feed(self):		
		for fd in self.feeds:
			if fd.active == 1:
				self.box.status_2("Checking " + fd.ticker + " ...",fd.iconcache, g.STOCK_CONVERT)
				yield None
				try:
					feed=urllib.urlopen(fd.ticker)
				except (IOError, EOFError):
					self.box.status_2("Checking " + fd.ticker + " ...",fd.iconcache, g.STOCK_CANCEL)
					yield tasks.TimeoutBlocker(2);
					continue
				try:
					xmldoc=minidom.parse(feed)
				except xml.sax.SAXParseException:
					self.box.status_2("Checking " + fd.ticker + " ...",fd.iconcache, g.STOCK_CANCEL)
					yield tasks.TimeoutBlocker(2);
					continue
				yield None
				self.read_xml(fd, xmldoc)
		if v: print "Refresh done (" + self.stat() +")"
		print "Refresh done         (" + self.stat() +")"
		self.export_xml()
		self.refresh_possible = True
		self.add_refresh()
