#!/usr/bin/env python
from __future__ import generators
# import findrox; findrox.version(1,9,12)
# it uses a patched version of ROX-Lib2 in rox/
import rox
from rox import g, tasks
from Options import *
import RETicker
import tool
import ticker_menu
import time

class StatusLine(g.HBox):
	def __init__(self, parent):
		g.HBox.__init__(self)
		self.con_parent = parent
		self.feed = parent.show_list.news.feeds[parent.feed_nr]
		self.set_image()
		text = self.feed.title
		if text == "": text = self.feed.ticker
		self.label=g.Label(text)
		self.label.set_size_request(-1,-1)
		self.label.set_alignment(0,0)
		self.label.set_padding(3,0)
		self.label.modify_fg(g.STATE_NORMAL,self.label.get_colormap().alloc_color(color_old_msg.value))
		self.label.show()
		self.add(self.label)
		self.modify_bg(g.STATE_NORMAL,self.get_colormap().alloc_color('#eee2b4'))
		self.show()
	def set_image(self):
		if self.feed.active == 1:
			self.i_act = tool.EventImage(g.STOCK_YES, self.deactivate, "active (click to change)")
		else:
			self.i_act = tool.EventImage(g.STOCK_NO, self.activate, "inactive (click to change)")
		self.i_act.set_small()
		self.i_act.modify_bg(g.STATE_NORMAL,self.i_act.get_colormap().alloc_color('#eee2b4'))
		self.pack_start(self.i_act, g.FALSE, g.FALSE, 0)
	def reset(self):
		self.remove(self.i_act)
		self.remove(self.label)
		del self.i_act
		self.set_image()
		self.add(self.label)
	def reinit(self, parent):
		self.feed = parent.show_list.news.feeds[parent.feed_nr]
		text = self.feed.title
		if text == "": text = self.feed.ticker
		self.label.set_text(text)
		self.reset()
	def activate(self, widget, event):
		self.feed.active = 1
		self.reset()
	def deactivate(self, widget, event):
		self.feed.active = 0
		self.reset()
	
		

class NavLine(g.HBox):
	def __init__(self, parent):
		g.HBox.__init__(self)
		i_prev = tool.EventImage(g.STOCK_GO_BACK, parent.prev_feed, "previous feed")
		i_next = tool.EventImage(g.STOCK_GO_FORWARD, parent.next_feed, "next feed")
		self.i_icon = tool.EventImage(g.STOCK_MISSING_IMAGE, self.open_feed_page, None)
		self.link = None
		self.pack_start(i_prev, g.FALSE, g.FALSE, 0)
		self.pack_start(self.i_icon, g.FALSE, g.FALSE, 0)
		self.pack_start(i_next, g.FALSE, g.FALSE, 0)
		hbox = g.HBox()
		i_window = tool.EventImage(g.STOCK_INDEX, self.window, "open all-feed window")
		i_refresh = tool.EventImage(g.STOCK_REFRESH, self.refresh, "refresh news")
		hbox.pack_start(i_refresh, g.FALSE, g.FALSE, 0)
		hbox.pack_start(i_window, g.FALSE, g.FALSE, 0)
		hbox.show()
		
		
		i_options = tool.EventImage(g.STOCK_PREFERENCES, self.options, "Preferences")
		i_help = tool.EventImage(g.STOCK_HELP, None, "Help")
		i_info = tool.EventImage(g.STOCK_DIALOG_INFO, None, parent.show_list.news.stat())
		i_quit = tool.EventImage(g.STOCK_QUIT, self.quit, "Exit NewsTicker")
		
		self.pack_end(i_quit, g.FALSE, g.FALSE, 10)
		self.pack_end(i_help, g.FALSE, g.FALSE, 0)
		self.pack_end(i_info, g.FALSE, g.FALSE, 0)
		self.pack_end(i_options, g.FALSE, g.FALSE, 0)
		self.pack_end(hbox, g.FALSE, g.FALSE, 10)
		
		i_prev.modify_bg(g.STATE_NORMAL,i_prev.get_colormap().alloc_color('#eee2b4'))
		self.i_icon.modify_bg(g.STATE_NORMAL,self.i_icon.get_colormap().alloc_color('#eee2b4'))
		i_next.modify_bg(g.STATE_NORMAL,i_next.get_colormap().alloc_color('#eee2b4'))
		i_window.modify_bg(g.STATE_NORMAL,i_window.get_colormap().alloc_color('#eee2b4'))
		i_refresh.modify_bg(g.STATE_NORMAL,i_refresh.get_colormap().alloc_color('#eee2b4'))
		i_options.modify_bg(g.STATE_NORMAL,i_options.get_colormap().alloc_color('#eee2b4'))
		i_help.modify_bg(g.STATE_NORMAL,i_help.get_colormap().alloc_color('#eee2b4'))
		i_info.modify_bg(g.STATE_NORMAL,i_info.get_colormap().alloc_color('#eee2b4'))
		i_quit.modify_bg(g.STATE_NORMAL,i_quit.get_colormap().alloc_color('#eee2b4'))
		
		self.modify_bg(g.STATE_NORMAL,self.get_colormap().alloc_color('#eee2b4'))
		
		self.show()
	def window(self, widget, event): tasks.Task(show_window())
	def refresh(self, widget, event): tasks.Task(RETicker.main.news.refresh_ticker())
	def options(self, widget, event): ticker_menu.show_options() ###
	def help(self, widget, event): ticker_menu.help() ###
	def quit(self, widget, event): ticker_menu.quit(None) ###
	def set_icon(self, Filename, link):
		self.i_icon.get_from_file(Filename)
		self.link = link
	def open_feed_page(self, widget, event):
		if self.link:
			if br_options.value <> "":
				os.spawnlp(os.P_NOWAIT,browser.value,browser.value,br_options.value,self.link) 
			else: 	os.spawnlp(os.P_NOWAIT,browser.value,browser.value,self.link)
	def set_tip(self, Tip):
		self.i_icon.set_tip(Tip)




class EventLabel(g.EventBox):
	def __init__(self, parent, msg, color, tip):
		g.EventBox.__init__(self)
		self.msg = msg
		self.con_parent = parent
		text = tool.unxml(self.msg.title).encode("utf-8")
		self.label=g.Label(text)
		self.label.set_size_request(-1,-1)
		self.label.set_alignment(0,0)
		self.label.set_padding(3,0)
		self.label.modify_fg(g.STATE_NORMAL,self.label.get_colormap().alloc_color(color))
		self.modify_bg(g.STATE_NORMAL,self.get_colormap().alloc_color('#eee2b4'))
		#self.modify_bg(g.STATE_NORMAL,self.get_colormap().alloc_color('white'))
		self.label.show()
		self.add(self.label)
		self.show()
		self.set_events(g.gdk.ENTER_NOTIFY_MASK | g.gdk.LEAVE_NOTIFY_MASK)
		self.connect("enter-notify-event", self.enter_widget)
		self.connect("leave-notify-event", self.leave_widget)
		self.connect("button_press_event",self.callback_event)
		self.tooltip = g.Tooltips()
		self.tooltip.enable()
		tiptext = text + " (" + tip +")"
		desc = tool.unxml(msg.desc.encode("utf-8"))
		if desc <> "(keine weitere Info)":
			tiptext += "\n---------------------\n" + desc
		self.tooltip.set_tip(self, tiptext)
		
	def enter_widget(self, widget, event):
		widget.set_state (g.STATE_PRELIGHT)
	def leave_widget(self, widget, event):
		widget.set_state (g.STATE_NORMAL)
	def callback_event(self, widget, event):
		if event.button == 1: tasks.Task(self.open_page())
        	if event.button == 2: self.con_show_list.update_ticker_()
		if event.button == 3: self.con_parent.open_menu(widget, event, None, self.msg)
	def open_desc_window(self):
		yield None
		desc_window = Desc_Window(self.msg, self.label)
	def open_page(self):
        	yield None
		self.modify_fg(g.STATE_NORMAL,self.get_colormap().alloc_color(color_old_msg.value))
		self.msg.read = 1
		yield None
		if br_options.value <> "":
			os.spawnlp(os.P_NOWAIT,browser.value,browser.value,br_options.value,self.msg.link) 
		else: 	os.spawnlp(os.P_NOWAIT,browser.value,browser.value,self.msg.link)

class ov_message(g.HBox):
	def __init__(self, parent, fd, i, count):
		g.HBox.__init__(self)
		self.msg = i
		color = color_old_msg.value
		if fd.msg_is_new(i): color = color_new_msg.value
		if count > fd.shownews: color = color_not_shown_msg.value
		if show_age.int_value == 1:
			page_time = i.fetchtime
			time_val = time.strftime("%a %H:%M", time.localtime(page_time))
			if i.time <> 0:	page_time = i.time; time_val += time.strftime(" [%a %H:%M]", time.localtime(page_time))
			#self.timeBox = EventLabel(time_val, color)
			#self.pack_start(self.timeBox, g.FALSE, g.FALSE, 0)
		if show_stat.int_value == 1:
			stat = "[" + str(i.showcount) + "x]"
			#self.countBox = EventLabel(stat, color)
			#self.pack_start(self.countBox, g.FALSE, g.FALSE, 0)
		self.message= EventLabel(parent, i, color, time_val + " " + stat)
		self.pack_start(self.message, g.FALSE, g.FALSE, 0)
		
		#self.modify_fg(g.STATE_NORMAL,
		#self.get_colormap().alloc_color('white'))
		#label.show()
		self.show()
		

class ov_feed(g.VBox):
	def __init__(self, parent, fd):
		g.VBox.__init__(self)
		count = 1
		self.con_parent = parent		
		if len(fd.msg) <> 0:
			for i in fd.msg:
				self.pack_start(ov_message(self, fd, i, count), g.FALSE, g.FALSE, 0)
				count += 1
		else:
			print "no msg"
			self.label1=g.Label("no messages available")
			self.label1.set_size_request(-1,-1)
			self.label1.set_alignment(0,0)
			self.label1.set_padding(3,0)
			self.label1.modify_fg(g.STATE_NORMAL,self.label1.get_colormap().alloc_color(color_old_msg.value))
			self.label1.show()
			self.label2=g.Label("maybe you have to\nactivate ticker or\nrefresh the news")
			self.label2.set_size_request(-1,-1)
			self.label2.set_alignment(0,0)
			self.label2.set_padding(3,0)
			self.label2.modify_fg(g.STATE_NORMAL,self.label1.get_colormap().alloc_color(color_not_shown_msg.value))
			#self.label2.set_line_wrap(g.TRUE)
			self.label2.show()
			#self.modify_bg(g.STATE_NORMAL,self.get_colormap().alloc_color('#eee2b4'))
			#self.modify_bg(g.STATE_NORMAL,self.get_colormap().alloc_color('white'))
			#label = g.Label("no messages available")
			#label = EventLabel(parent, i, color, time_val + " " + stat)
			#label.show
			self.pack_start(self.label1, g.FALSE, g.FALSE, 0)
			self.add(self.label2)
		self.show()
		from rox.Menu import Menu, set_save_name
		self.menu = Menu('ov_aktfeed', [
        		('/Quit',		'quit',		'<StockItem>',  '', g.STOCK_QUIT),
 			('/Help',          	'help',         '<StockItem>',  '', g.STOCK_HELP),
        		('/Info',          	'info',         '<StockItem>',  '', g.STOCK_DIALOG_INFO),
        		('/Show Window',        'show_window',  '<StockItem>',  '', g.STOCK_INDEX),
			('/Options',            'show_options', '<StockItem>',  '', g.STOCK_PREFERENCES),
			('/Refresh',            'refresh_ticker', '<StockItem>',  '', g.STOCK_REFRESH),
        		('/Next Msg',            'update_ticker', '<StockItem>',  '', g.STOCK_GO_FORWARD)
        		])
		self.menu.menu.set_events(g.gdk.DELETE)
		self.menu.menu.connect("deactivate", self.menu_closed)
		#self.menu.menu.connect("leave-notify-event", self.leave_widget)
	def menu_closed(self, event):
		print "menu closed"
		#self.con_parent.Menu = None
		#tasks.Task(self.con_parent.wait_and_hide())
	def enter_widget(self, widget, event):
		print "enter menu"
		#self.active = True
		#widget.set_state (g.STATE_PRELIGHT)
	def leave_widget(self, widget, event):
		print "leave menu"
		#if not(self.active): return
		#else: self.hide_now()
		#widget.set_state (g.STATE_NORMAL)
	def open_menu(self, widget, event, position_menu, msg):
		self.msg = msg
		#self.con_parent.Menu = 1
		#print "bevor menu"
		#self.menu.popup(widget, event, position_menu)
		#print "menu done"
		#self.con_parent.Menu = None
		#tasks.Task(self.con_parent.wait_and_hide())
		
def show_window_2(main):
	ov_aktfeed(main)
	yield None
	
class ov_aktfeed:
	def __init__(self):
		self.initialized = False
		self.active = False
		self.feed_nr = 0
		self.show_list = None
	def next_feed(self, widget, event):
		self.feed_nr = self.show_list.next_feed(self.feed_nr)
		self.refresh()
	def prev_feed(self, widget, event):
		self.feed_nr = self.show_list.prev_feed(self.feed_nr)
		self.refresh()
	def akt_feed(self):
		yield None
		if self.active: return
		self.feed_nr = self.show_list.fd_count
		self.refresh()
	def refresh(self):	
		self.ebox2.remove(self.overview)
		del self.overview
		feed = self.show_list.news.feeds[self.feed_nr]
		self.overview = ov_feed(self, feed)
		self.ebox2.add(self.overview)
		self.nav.set_icon(feed.iconcache, tool.unxml(feed.link).encode("ascii"))
		self.nav.set_tip(tool.unxml(feed.title).encode("utf-8") + "\n" + \
				tool.unxml(feed.desc).encode("utf-8"))
		
		self.stat.reinit(self)
	def create_window(self, show_list):
		#print "create window"
		self.feed_nr = show_list.fd_count
		self.show_list = show_list
		self.window=g.Window()
		self.window.set_decorated(g.FALSE)
		self.window.modify_bg(g.STATE_NORMAL,self.window.get_colormap().alloc_color('black'))
		frame=g.Frame()
		frame.show()
		#frame.set_border_width(0)
		#frame.set_shadow_type(g.SHADOW_NONE)
		frame.modify_bg(g.STATE_NORMAL,frame.get_colormap().alloc_color('black'))
		#frame.modify_fg(g.STATE_NORMAL,frame.get_colormap().alloc_color('green'))
		ebox = g.EventBox() # without thisone the enter and leave-widget events dont work
		ebox.modify_bg(g.STATE_NORMAL,ebox.get_colormap().alloc_color('#eee2b4'))
		ebox.show()
		vbox = g.VBox()
		#vbox.modify_bg(g.STATE_NORMAL,vbox.get_colormap().alloc_color('#eee2b4'))
		vbox.show()

		swin=g.ScrolledWindow()
		swin.set_shadow_type(g.SHADOW_NONE)
		#swin.set_shadow_type(g.SHADOW_IN)
		swin.set_policy(g.POLICY_NEVER, g.POLICY_AUTOMATIC)
		swin.show()
		
		self.ebox2 = g.EventBox()
		self.ebox2.modify_bg(g.STATE_NORMAL,self.ebox2.get_colormap().alloc_color('#eee2b4'))
		#self.ebox2.modify_bg(g.STATE_NORMAL,self.ebox2.get_colormap().alloc_color('white'))
		self.ebox2.show()
		
		feed = show_list.akt_feed()
		self.overview = ov_feed(self, feed)
		self.ebox2.add(self.overview)
		#self.ebox2.set_shadow_type(g.SHADOW_NONE)
		swin.add_with_viewport(self.ebox2)
		self.stat = StatusLine(self)
		
		self.nav = NavLine(self)
		self.nav.set_icon(feed.iconcache, tool.unxml(feed.link).encode("ascii"))
		self.nav.set_tip(tool.unxml(feed.title).encode("utf-8") + "\n" + \
				tool.unxml(feed.desc).encode("utf-8"))
		vbox.pack_start(self.stat, g.FALSE, g.FALSE, 0)
		vbox.pack_end(self.nav, g.FALSE, g.FALSE, 0)
		vbox.add(swin)
		frame.add(vbox)
		ebox.add(frame)
		self.window.add(ebox)
		self.window.set_size_request(200,200)
		#self.window.set_size_request(-1,-1)
		self.window.set_events(g.gdk.ENTER_NOTIFY_MASK | g.gdk.LEAVE_NOTIFY_MASK)
		self.window.connect("enter-notify-event", self.enter_widget)
		self.window.connect("leave-notify-event", self.leave_widget)
		self.initialized = True
		self.Menu = None
	def enter_widget(self, widget, event):
		#print "enter wdget"
		self.active = True
		#widget.set_state (g.STATE_PRELIGHT)
	def leave_widget(self, widget, event):
		#print "leave widget"
		if not(self.active):
			#print "keine Ahnung wann das passiert"
			return
		else: 
			if not(self.Menu): self.hide_now()
		#widget.set_state (g.STATE_NORMAL)
	def show_now(self, x, y, w, h):	
		if self.initialized:
			#print "show window"
			scr_w = g.gdk.screen_width()
			scr_h = g.gdk.screen_height()
			width, height = self.window.size_request()
			# try over main window
			if y - height >= 0: y = y - height
			else:	y = y + h
			if x < 0 : x = 0
			if x + width > scr_w: x = scr_w - width
			self.window.move(x, y)
			
			self.window.show()
			#print "done"
	def hide_now(self):
		#print "hide now"
		self.active = False
		tasks.Task(self.wait_and_hide())
	def wait_and_hide(self):
		yield tasks.TimeoutBlocker(0.3);
		if not(self.active) and self.initialized:
			#print "hide window"
			self.window.hide()
			self.feed_nr = self.show_list.fd_count
			self.refresh()
		

""" a little python script to get rss feeds and display them """
################################################################
######## imported from a python script called ticker.py ########
################################################################
def go_(self, event, address):
	if br_options.value <> "":
		os.spawnlp(os.P_NOWAIT,browser.value,browser.value,br_options.value,address) 
	else: 	os.spawnlp(os.P_NOWAIT,browser.value,browser.value,address)

def go(self, msg, label):
        label.modify_fg(g.STATE_NORMAL,label.get_colormap().alloc_color(color_old_msg.value))
	msg.read = 1
	if br_options.value <> "":
		os.spawnlp(os.P_NOWAIT,browser.value,browser.value,br_options.value,msg.link.encode("ascii")) 
	else: 	os.spawnlp(os.P_NOWAIT,browser.value,browser.value,msg.link.encode("ascii"))

def description_window(dummy,dict,label):
	""" This function opens an additional Window with a text box showing
	the title, the description and a link """
	dwin = g.Window()
	swin = g.ScrolledWindow()
	vbox = g.VBox(spacing = 4)
	hbox = g.HBox()
	title = g.Label(tool.unxml(dict.title).encode("utf-8"))
	link = g.Button("Link")
	d_buffer = g.TextBuffer();
	description=g.TextView(d_buffer) 

	# Destroy is implied... I think...
	# dwin.connect("destroy", self.destroy)
	dwin.set_title(dict.title.encode("utf-8"))
	dwin.set_border_width(3)
	dwin.add(vbox)
	vbox.pack_start(hbox, False, False)
	vbox.add(swin)
	swin.add_with_viewport(description)
	description.set_editable(g.FALSE)
	description.set_cursor_visible(g.FALSE)
	description.set_wrap_mode(g.WRAP_WORD)
	#swin.set_size_request(150,100)
	swin.set_size_request(350,100)

	hbox.pack_start(title, expand=g.FALSE, fill=g.TRUE, padding = 2)
	hbox.pack_end(link, expand=False, fill=False, padding = 4)
	link.connect("clicked", go, dict, label)
	swin.set_policy(g.POLICY_AUTOMATIC, g.POLICY_AUTOMATIC)
	d_buffer.set_text(tool.unxml(dict.desc.encode("utf-8")));

	dwin.show_all();


def callback_window(widget, event, dict, label):
	if event.button == 1:
		go(widget, dict, label)
	if event.button == 3:
		description_window(event, dict, label)

def emptybox(box):
	
	for child in box.get_children():
		box.remove(child)
	return box
	
def show_it(vbox):
	""" this function shows all items """
	for fd in RETicker.main.news.feeds:
		if fd.active <> 1: continue
		title = fd.title
		link  = fd.link
		desc  = fd.desc
		ticker = ""
		sep=g.HSeparator()
		sep.show()
		vbox.pack_start(sep,g.FALSE,g.FALSE,0)
		vbox2 = g.VBox()
		vbox2.show()
		label_t=g.Label(tool.unxml(title).encode("utf-8"))
		label_t.set_alignment(0,0)
		label_t.show()
		if not(fd.valid):
			label_t.modify_fg(g.STATE_NORMAL,label_t.get_colormap().alloc_color('red'))
		label_d=g.Label(tool.unxml(desc).encode("utf-8"))
		label_d.set_alignment(0,0)
		label_d.show()
		vbox2.pack_start(label_t, g.FALSE, g.FALSE, 0)
		vbox2.pack_start(label_d, g.FALSE, g.FALSE, 0)
		#image = RETicker.TickBild()
		image = tool.EventImage(g.STOCK_MISSING_IMAGE, None,None)
		image.modify_bg(g.STATE_NORMAL,image.get_colormap().alloc_color('white'))
		image.get_from_file(fd.iconcache)
		hbox = g.HBox()
		hbox.show()
		hbox.pack_start(image, g.FALSE, g.FALSE, 3)
		hbox.pack_start(vbox2, g.FALSE, g.FALSE, 3)
		ebox=g.EventBox()
		ebox.modify_bg(g.STATE_NORMAL,ebox.get_colormap().alloc_color('white'))
		ebox.modify_bg(g.STATE_PRELIGHT,ebox.get_colormap().alloc_color('red'))
		ebox.add(hbox)
		ebox.connect("button_press_event",go_,link.encode("ascii"))
		ebox.show_all()
		vbox.pack_start(ebox,g.FALSE,g.FALSE,0)
		count = 1
		for i in fd.msg:
			ebox=g.EventBox()
			#tooltip=g.Tooltips()
			page_time = 0
			if show_age.int_value == 1:
				page_time = i.fetchtime
				time_val = time.strftime("fetched %a %H:%M ", time.localtime(page_time))
				if i.time <> 0:	page_time = i.time; time_val += time.strftime("[%a %H:%M] ", time.localtime(page_time)) + " "
			else: 	time_val = ""
			if show_stat.int_value == 1:
				stat = "[" + str(i.showcount) + "x] "
			else: stat = ""
			label=g.Label(time_val + stat + tool.unxml(i.title).encode("utf-8"))
			label.set_alignment(0,0)
			label.modify_fg(g.STATE_NORMAL,label.get_colormap().alloc_color(color_old_msg.value))
			if fd.msg_is_new(i):
				label.modify_fg(g.STATE_NORMAL,label.get_colormap().alloc_color(color_new_msg.value))
			if count > fd.shownews:
				label.modify_fg(g.STATE_NORMAL,label.get_colormap().alloc_color(color_not_shown_msg.value))
			ebox.add(label)
			ebox.connect("button_press_event",callback_window,i,label)
			ebox.modify_bg(g.STATE_NORMAL,
			ebox.get_colormap().alloc_color('white'))
			#tooltip.set_tip(ebox,i['link'].encode("ascii"))
			label.show()
			ebox.show()
			vbox.pack_start(ebox,g.FALSE,g.FALSE,0)
			count += 1
	while g.events_pending():                                             
	            g.mainiteration(g.FALSE)

def update(event):
	global wbox
	global swin
	wbox.destroy()
	wbox=g.VBox()
	wbox.show()
	swin.add_with_viewport(wbox)
	show_it(wbox)
	global stat_label
	stat_label.set_text(RETicker.main.news.stat())

def show_window():
	window=g.Window()
	vbox = g.VBox()
	global wbox
	global swin
	global stat_label
	stat_label = g.Label(RETicker.main.news.stat())
	stat_label.show()
	wbox=g.VBox()
	frame=g.Frame()
	swin=g.ScrolledWindow()
	window.set_title(" " + Version + " ")
	window.set_border_width(5)
	window.set_resizable(g.TRUE)
	window.connect("destroy", destroy)
	window.add(vbox)
	vbox.add(frame)
	if show_stat.int_value == 1:
		hbox = g.HBox()
		button1 = g.Button("update Window")
		button1.show()
		button1.set_size_request(90,18)
		button1.connect('clicked', update)
		button2 = g.Button("refresh News")
		button2.show()
		button2.set_size_request(90,18)
		def refresh(event):
			import RETicker
			RETicker.main.news.start_refresh_ticker_task()
		button2.connect('clicked', refresh)
		hbox.show()
		hbox.add(button1)
		hbox.add(button2)
		hbox.add(stat_label)
		vbox.add(hbox)
	vbox.show()
	frame.add(swin)
	frame.show()
	wbox.show()
	swin.add_with_viewport(wbox)
	swin.set_size_request(500,500)
	swin.modify_fg(g.STATE_NORMAL,
	swin.get_colormap().alloc_color('white'))
	swin.set_policy(g.POLICY_AUTOMATIC,g.POLICY_AUTOMATIC)
	swin.show()
	yield None
	show_it(wbox)
	window.show()

def destroy(self):
       g.main_quit()

################################################################
################################################################


